@@ -1,11 +1,15 @@
+-file("xmerl_b64Bin.yrl", 0).
 -module(xmerl_b64Bin).
+-file("xmerl_b64Bin.erl", 3).
 -export([parse/1, parse_and_scan/1, format_error/1]).
+-file("xmerl_b64Bin.yrl", 71).
+-moduledoc false.
 
 -file("/build/erlang/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,10 +31,16 @@
 
 -type yecc_ret() :: {'error', _} | {'ok', _}.
 
+-ifdef (YECC_PARSE_DOC).
+-doc ?YECC_PARSE_DOC.
+-endif.
 -spec parse(Tokens :: list()) -> yecc_ret().
 parse(Tokens) ->
     yeccpars0(Tokens, {no_func, no_location}, 0, [], []).
 
+-ifdef (YECC_PARSE_AND_SCAN_DOC).
+-doc ?YECC_PARSE_AND_SCAN_DOC.
+-endif.
 -spec parse_and_scan({function() | {atom(), atom()}, [_]}
                      | {atom(), atom(), [_]}) -> yecc_ret().
 parse_and_scan({F, A}) ->
@@ -39,6 +49,9 @@ parse_and_scan({M, F, A}) ->
     Arity = length(A),
     yeccpars0([], {{fun M:F/Arity, A}, no_location}, 0, [], []).
 
+-ifdef (YECC_FORMAT_ERROR_DOC).
+-doc ?YECC_FORMAT_ERROR_DOC.
+-endif.
 -spec format_error(any()) -> [char() | list()].
 format_error(Message) ->
     case io_lib:deep_char_list(Message) of
@@ -152,28 +165,38 @@ yecctoken_location(Token) ->
     end.
 
 -compile({nowarn_unused_function, yecctoken2string/1}).
-yecctoken2string({atom, _, A}) -> io_lib:write_atom(A);
-yecctoken2string({integer,_,N}) -> io_lib:write(N);
-yecctoken2string({float,_,F}) -> io_lib:write(F);
-yecctoken2string({char,_,C}) -> io_lib:write_char(C);
-yecctoken2string({var,_,V}) -> io_lib:format("~s", [V]);
-yecctoken2string({string,_,S}) -> io_lib:write_string(S);
-yecctoken2string({reserved_symbol, _, A}) -> io_lib:write(A);
-yecctoken2string({_Cat, _, Val}) -> io_lib:format("~tp", [Val]);
-yecctoken2string({dot, _}) -> "'.'";
-yecctoken2string({'$end', _}) -> [];
-yecctoken2string({Other, _}) when is_atom(Other) ->
+yecctoken2string(Token) ->
+    try
+        yecctoken2string1(Token)
+    catch
+        _:_ ->
+            io_lib:format("~tp", [Token])
+    end.
+
+-compile({nowarn_unused_function, yecctoken2string1/1}).
+yecctoken2string1({atom, _, A}) -> io_lib:write_atom(A);
+yecctoken2string1({integer,_,N}) -> io_lib:write(N);
+yecctoken2string1({float,_,F}) -> io_lib:write(F);
+yecctoken2string1({char,_,C}) -> io_lib:write_char(C);
+yecctoken2string1({var,_,V}) -> io_lib:format("~s", [V]);
+yecctoken2string1({string,_,S}) -> io_lib:write_string(S);
+yecctoken2string1({reserved_symbol, _, A}) -> io_lib:write(A);
+yecctoken2string1({_Cat, _, Val}) -> io_lib:format("~tp", [Val]);
+yecctoken2string1({dot, _}) -> "'.'";
+yecctoken2string1({'$end', _}) -> [];
+yecctoken2string1({Other, _}) when is_atom(Other) ->
     io_lib:write_atom(Other);
-yecctoken2string(Other) ->
+yecctoken2string1(Other) ->
     io_lib:format("~tp", [Other]).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
 
--file("xmerl_b64Bin.erl", 174).
+-file("xmerl_b64Bin.erl", 196).
 
 -dialyzer({nowarn_function, yeccpars2/7}).
+-compile({nowarn_unused_function,  yeccpars2/7}).
 yeccpars2(0=S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(S, Cat, Ss, Stack, T, Ts, Tzr);
 %% yeccpars2(1=S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -213,58 +236,73 @@ yeccpars2(15=S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccpars2(Other, _, _, _, _, _, _) ->
  erlang:error({yecc_bug,"1.4",{missing_state_in_action_table, Other}}).
 
-yeccpars2_0(S, b04, Ss, Stack, T, Ts, Tzr) ->
+-dialyzer({nowarn_function, yeccpars2_0/7}).
+-compile({nowarn_unused_function,  yeccpars2_0/7}).
+yeccpars2_0(S, 'b04', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 4, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, b16x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'b16x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 5, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, b64x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'b64x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
 yeccpars2_0(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_0_(Stack),
  yeccpars2_2(2, Cat, [0 | Ss], NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_1/7}).
+-compile({nowarn_unused_function,  yeccpars2_1/7}).
 yeccpars2_1(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_1_(Stack),
  yeccgoto_base64Binary(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccpars2_2/7}).
+-compile({nowarn_unused_function,  yeccpars2_2/7}).
 yeccpars2_2(_S, '$end', _Ss, Stack, _T, _Ts, _Tzr) ->
  {ok, hd(Stack)};
 yeccpars2_2(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
 -dialyzer({nowarn_function, yeccpars2_3/7}).
-yeccpars2_3(S, b04, Ss, Stack, T, Ts, Tzr) ->
+-compile({nowarn_unused_function,  yeccpars2_3/7}).
+yeccpars2_3(S, 'b04', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 8, Ss, Stack, T, Ts, Tzr);
-yeccpars2_3(S, b16x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_3(S, 'b16x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 5, Ss, Stack, T, Ts, Tzr);
-yeccpars2_3(S, b64x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_3(S, 'b64x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
 yeccpars2_3(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
+-dialyzer({nowarn_function, yeccpars2_4/7}).
+-compile({nowarn_unused_function,  yeccpars2_4/7}).
 yeccpars2_4(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_4_(Stack),
  yeccgoto_b64(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_5/7}).
+-compile({nowarn_unused_function,  yeccpars2_5/7}).
 yeccpars2_5(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_5_(Stack),
  yeccgoto_b64(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_6/7}).
+-compile({nowarn_unused_function,  yeccpars2_6/7}).
 yeccpars2_6(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_6_(Stack),
  yeccgoto_b64(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccpars2_7/7}).
-yeccpars2_7(S, b04, Ss, Stack, T, Ts, Tzr) ->
+-compile({nowarn_unused_function,  yeccpars2_7/7}).
+yeccpars2_7(S, 'b04', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 13, Ss, Stack, T, Ts, Tzr);
-yeccpars2_7(S, b16x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_7(S, 'b16x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 14, Ss, Stack, T, Ts, Tzr);
-yeccpars2_7(S, b64x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_7(S, 'b64x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
 yeccpars2_7(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
+-dialyzer({nowarn_function, yeccpars2_8/7}).
+-compile({nowarn_unused_function,  yeccpars2_8/7}).
 yeccpars2_8(S, '=', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 9, Ss, Stack, T, Ts, Tzr);
 yeccpars2_8(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -272,72 +310,89 @@ yeccpars2_8(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_b64(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccpars2_9/7}).
+-compile({nowarn_unused_function,  yeccpars2_9/7}).
 yeccpars2_9(S, '=', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 10, Ss, Stack, T, Ts, Tzr);
 yeccpars2_9(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
+-dialyzer({nowarn_function, yeccpars2_10/7}).
+-compile({nowarn_unused_function,  yeccpars2_10/7}).
 yeccpars2_10(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  [_,_,_|Nss] = Ss,
  NewStack = yeccpars2_10_(Stack),
  yeccgoto_base64Binary2(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccpars2_11/7}).
-yeccpars2_11(S, b04, Ss, Stack, T, Ts, Tzr) ->
+-compile({nowarn_unused_function,  yeccpars2_11/7}).
+yeccpars2_11(S, 'b04', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 4, Ss, Stack, T, Ts, Tzr);
-yeccpars2_11(S, b16x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_11(S, 'b16x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 5, Ss, Stack, T, Ts, Tzr);
-yeccpars2_11(S, b64x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_11(S, 'b64x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
 yeccpars2_11(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
 -dialyzer({nowarn_function, yeccpars2_12/7}).
+-compile({nowarn_unused_function,  yeccpars2_12/7}).
 yeccpars2_12(S, '=', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 15, Ss, Stack, T, Ts, Tzr);
 yeccpars2_12(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
+-dialyzer({nowarn_function, yeccpars2_13/7}).
+-compile({nowarn_unused_function,  yeccpars2_13/7}).
 yeccpars2_13(_S, '=', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_13_\'=\''(Stack),
+ NewStack = 'yeccpars2_13_='(Stack),
  yeccgoto_b16(hd(Ss), '=', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_13(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_13_(Stack),
  yeccgoto_b64(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_14/7}).
+-compile({nowarn_unused_function,  yeccpars2_14/7}).
 yeccpars2_14(_S, '=', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_14_\'=\''(Stack),
+ NewStack = 'yeccpars2_14_='(Stack),
  yeccgoto_b16(hd(Ss), '=', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_14(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_14_(Stack),
  yeccgoto_b64(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_15/7}).
+-compile({nowarn_unused_function,  yeccpars2_15/7}).
 yeccpars2_15(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  [_,_,_|Nss] = Ss,
  NewStack = yeccpars2_15_(Stack),
  yeccgoto_base64Binary2(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
-yeccpars2_16(S, b04, Ss, Stack, T, Ts, Tzr) ->
+-dialyzer({nowarn_function, yeccpars2_16/7}).
+-compile({nowarn_unused_function,  yeccpars2_16/7}).
+yeccpars2_16(S, 'b04', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 4, Ss, Stack, T, Ts, Tzr);
-yeccpars2_16(S, b16x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_16(S, 'b16x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 5, Ss, Stack, T, Ts, Tzr);
-yeccpars2_16(S, b64x, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_16(S, 'b64x', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
 yeccpars2_16(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  [_,_,_|Nss] = Ss,
  NewStack = yeccpars2_16_(Stack),
  yeccgoto_base64Binary2(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_17/7}).
+-compile({nowarn_unused_function,  yeccpars2_17/7}).
 yeccpars2_17(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  [_,_,_,_|Nss] = Ss,
  NewStack = yeccpars2_17_(Stack),
  yeccgoto_base64Binary2(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccgoto_b16/7}).
+-compile({nowarn_unused_function,  yeccgoto_b16/7}).
 yeccgoto_b16(7, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_12(12, Cat, Ss, Stack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccgoto_b64/7}).
+-compile({nowarn_unused_function,  yeccgoto_b64/7}).
 yeccgoto_b64(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_3(3, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_b64(3, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -350,16 +405,20 @@ yeccgoto_b64(16, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_3(3, Cat, Ss, Stack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccgoto_base64Binary/7}).
+-compile({nowarn_unused_function,  yeccgoto_base64Binary/7}).
 yeccgoto_base64Binary(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_2(2, Cat, Ss, Stack, T, Ts, Tzr).
 
 -dialyzer({nowarn_function, yeccgoto_base64Binary2/7}).
+-compile({nowarn_unused_function,  yeccgoto_base64Binary2/7}).
 yeccgoto_base64Binary2(0=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_1(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_base64Binary2(16=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_17(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
 -compile({inline,yeccpars2_0_/1}).
+-dialyzer({nowarn_function, yeccpars2_0_/1}).
+-compile({nowarn_unused_function,  yeccpars2_0_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_0_(__Stack0) ->
  [begin
@@ -367,6 +426,8 @@ yeccpars2_0_(__Stack0) ->
   end | __Stack0].
 
 -compile({inline,yeccpars2_1_/1}).
+-dialyzer({nowarn_function, yeccpars2_1_/1}).
+-compile({nowarn_unused_function,  yeccpars2_1_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_1_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -375,6 +436,8 @@ yeccpars2_1_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_4_/1}).
+-dialyzer({nowarn_function, yeccpars2_4_/1}).
+-compile({nowarn_unused_function,  yeccpars2_4_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_4_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -383,6 +446,8 @@ yeccpars2_4_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_5_/1}).
+-dialyzer({nowarn_function, yeccpars2_5_/1}).
+-compile({nowarn_unused_function,  yeccpars2_5_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_5_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -391,6 +456,8 @@ yeccpars2_5_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_6_/1}).
+-dialyzer({nowarn_function, yeccpars2_6_/1}).
+-compile({nowarn_unused_function,  yeccpars2_6_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_6_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -399,6 +466,8 @@ yeccpars2_6_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_8_/1}).
+-dialyzer({nowarn_function, yeccpars2_8_/1}).
+-compile({nowarn_unused_function,  yeccpars2_8_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_8_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -407,6 +476,8 @@ yeccpars2_8_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_10_/1}).
+-dialyzer({nowarn_function, yeccpars2_10_/1}).
+-compile({nowarn_unused_function,  yeccpars2_10_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_10_(__Stack0) ->
  [___4,___3,___2,___1 | __Stack] = __Stack0,
@@ -414,15 +485,19 @@ yeccpars2_10_(__Stack0) ->
 '$undefined'
   end | __Stack].
 
--compile({inline,'yeccpars2_13_\'=\''/1}).
+-compile({inline,'yeccpars2_13_='/1}).
+-dialyzer({nowarn_function, 'yeccpars2_13_='/1}).
+-compile({nowarn_unused_function,  'yeccpars2_13_='/1}).
 -file("xmerl_b64Bin.yrl", 0).
-'yeccpars2_13_\'=\''(__Stack0) ->
+'yeccpars2_13_='(__Stack0) ->
  [___1 | __Stack] = __Stack0,
  [begin
 '$undefined'
   end | __Stack].
 
 -compile({inline,yeccpars2_13_/1}).
+-dialyzer({nowarn_function, yeccpars2_13_/1}).
+-compile({nowarn_unused_function,  yeccpars2_13_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_13_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -430,15 +505,19 @@ yeccpars2_13_(__Stack0) ->
 '$undefined'
   end | __Stack].
 
--compile({inline,'yeccpars2_14_\'=\''/1}).
+-compile({inline,'yeccpars2_14_='/1}).
+-dialyzer({nowarn_function, 'yeccpars2_14_='/1}).
+-compile({nowarn_unused_function,  'yeccpars2_14_='/1}).
 -file("xmerl_b64Bin.yrl", 0).
-'yeccpars2_14_\'=\''(__Stack0) ->
+'yeccpars2_14_='(__Stack0) ->
  [___1 | __Stack] = __Stack0,
  [begin
 '$undefined'
   end | __Stack].
 
 -compile({inline,yeccpars2_14_/1}).
+-dialyzer({nowarn_function, yeccpars2_14_/1}).
+-compile({nowarn_unused_function,  yeccpars2_14_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_14_(__Stack0) ->
  [___1 | __Stack] = __Stack0,
@@ -447,6 +526,8 @@ yeccpars2_14_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_15_/1}).
+-dialyzer({nowarn_function, yeccpars2_15_/1}).
+-compile({nowarn_unused_function,  yeccpars2_15_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_15_(__Stack0) ->
  [___4,___3,___2,___1 | __Stack] = __Stack0,
@@ -455,6 +536,8 @@ yeccpars2_15_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_16_/1}).
+-dialyzer({nowarn_function, yeccpars2_16_/1}).
+-compile({nowarn_unused_function,  yeccpars2_16_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_16_(__Stack0) ->
  [___4,___3,___2,___1 | __Stack] = __Stack0,
@@ -463,6 +546,8 @@ yeccpars2_16_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_17_/1}).
+-dialyzer({nowarn_function, yeccpars2_17_/1}).
+-compile({nowarn_unused_function,  yeccpars2_17_/1}).
 -file("xmerl_b64Bin.yrl", 0).
 yeccpars2_17_(__Stack0) ->
  [___5,___4,___3,___2,___1 | __Stack] = __Stack0,
@@ -471,3 +556,4 @@ yeccpars2_17_(__Stack0) ->
   end | __Stack].
 
 
+-file("xmerl_b64Bin.yrl", 73).
